﻿using Microsoft.EntityFrameworkCore;
using SushiBarContracts.BindingModels;
using SushiBarContracts.SearchModels;
using SushiBarContracts.StoragesContracts;
using SushiBarContracts.ViewModels;
using SushiBarDatabaseImplement.Models;

namespace SushiBarDatabaseImplement.Implements
{
    public class ClientStorage : IClientStorage
    {
        public List<ClientViewModel> GetFullList()
        {
            using var context = new SushiBarDatabase();
            return context.Clients
                .Select(c => c.GetViewModel)
                .ToList();
        }
        public List<ClientViewModel> GetFilteredList(ClientSearchModel model)
        {
            using var context = new SushiBarDatabase();
            return context.Clients
                .Where(c =>
                    (model.Id.HasValue && c.Id == model.Id) ||
                    (!string.IsNullOrEmpty(model.ClientFIO) && model.ClientFIO == c.ClientFIO) ||
                    (!string.IsNullOrEmpty(model.Email) && model.Email == c.Email))
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public ClientViewModel? GetElement(ClientSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new SushiBarDatabase();
            return context.Clients
                .FirstOrDefault(o => o.Id == model.Id)?
                .GetViewModel;
        }
        public ClientViewModel? Insert(ClientBindingModel model)
        {
            using var context = new SushiBarDatabase();
            var newClient = Client.Create(model);
            if (newClient == null)
            {
                return null;
            }
            context.Clients.Add(newClient);
            context.SaveChanges();
            return newClient.GetViewModel;
        }
        public ClientViewModel? Update(ClientBindingModel model)
        {
            using var context = new SushiBarDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var client = context.Clients
                    .FirstOrDefault(c => c.Id == model.Id);
                if (client == null)
                {
                    return null;
                }
                client.Update(model);
                context.SaveChanges();
                transaction.Commit();
                return client.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }
        public ClientViewModel? Delete(ClientBindingModel model)
        {
            using var context = new SushiBarDatabase();
            var element = context.Clients
                .FirstOrDefault(c => c.Id == model.Id);
            if (element != null)
            {
                context.Clients.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
